#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"


#define SCREEN_WIDTH    600
#define SCREEN_HEIGHT   600

CBlob::CBlob()
{
    pos.x = (rand() % SCREEN_WIDTH) * 256;
    pos.y = (rand() % SCREEN_HEIGHT) * 256;

    vel.x = (((rand() % 61) - 30) * 256) / 40;
    vel.y = (((rand() % 61) - 30) * 256) / 40;

    radius = 4000 + (rand() % 12000);
    radius *= 30;
}

void CBlob::update()
{
    pos.x += vel.x;
    pos.y += vel.y;

    if (pos.x < 0)
    {
        pos.x = 0;
        vel.x = -vel.x;
    }

    if (pos.x > (SCREEN_WIDTH - 1) * 256)
    {
        pos.x = (SCREEN_WIDTH - 1) * 256;
        vel.x = -vel.x;
    }

    if (pos.y < 0)
    {
        pos.y = 0;
        vel.y = -vel.y;
    }

    if (pos.y > (SCREEN_HEIGHT - 1) * 256)
    {
        pos.y = (SCREEN_HEIGHT - 1) * 256;
        vel.y = -vel.y;
    }
}

int main(int argc, char* argv[])
{
    // init the window
    gfx.init("Metaballs 13", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    srand(time(NULL));

    const int nbBlobs = 6;
    CBlob* blobs = new CBlob[nbBlobs];

    static Color palette[256];
    for (int i = 0; i < 256; i++)
    {
        float angle = (pow(i, 0.64) + 78) * M_PI / 20;
        palette[i].r = 127 * (1 + sin(angle));
        palette[i].g = 127 * (1 + sin(angle + 2 * M_PI / 3));
        palette[i].b = 127 * (1 + sin(angle + 4 * M_PI / 3));
    }

    static int div[nbBlobs][2048];

    for (int i = 0; i < nbBlobs; i++)
    {
        div[i][0] = 255;

        for (int j = 1; j < 2048; j++)
            div[i][j] = (blobs[i].radius / 128) / j;
    }

    while (sys.isQuitRequested() == false)
    {
        sys.StartPerfCounter();

        static int bX[nbBlobs];
        static int bY[nbBlobs];

        for (int i = 0; i < nbBlobs; i++)
        {
            bX[i] = (blobs[i].pos.x + 128) / 256;
            bY[i] = (blobs[i].pos.y + 128) / 256;
        }

        for (int y = 0; y < SCREEN_HEIGHT; y++)
            for (int x = 0; x < SCREEN_HEIGHT; x++)
            {
                int power = 0;

                for (int i = 0; i < nbBlobs; i++)
                {
                    int dX = x - bX[i];
                    int dY = y - bY[i];
                    int dist = (dX * dX + dY * dY) / 128;

                    if (dist < 2048)
                        power += div[i][dist];
                }

                if (power > 255)
                    power = 255;

                gfx.setPixel(x, y, palette[power]);
            }

        for (int i = 0; i < nbBlobs; i++)
            blobs[i].update();

        float tim = sys.StopPerfCounter();
        printf("%f\n", tim);

        gfx.render();
        sys.processEvents();
    }

    delete[] blobs;
    gfx.quit();

    return EXIT_SUCCESS;
}
